@@ -14,7 +14,7 @@ import struct
 import sys
 import sysconfig
 import time
-from typing import Any, Dict, List, Set, Tuple, Optional, Union
+from typing import List, Set, Tuple, Optional, Union
 import zipfile
 
 from .common import get_resource_file_path, process_path_specs
@@ -23,6 +23,7 @@ from .exception import ConfigError
 from .executable import Executable
 from .finder import ModuleFinder
 from .module import ConstantsModule, Module
+from .parser import Parser, ELFParser, PEParser
 
 DARWIN = sys.platform == "darwin"
 MINGW = sysconfig.get_platform().startswith("mingw")
@@ -30,17 +31,14 @@ WIN32 = sys.platform == "win32"
 
 if WIN32:
     from . import winmsvcr
-    from . import util as winutil
     from .winversioninfo import VersionInfo
 
     try:
-        from win32verstamp import stamp as version_stamp
+        from .util import AddIcon, GetWindowsDir, GetSystemDir, UpdateCheckSum
     except ImportError:
-        version_stamp = None
+        pass
 elif DARWIN:
     from .darwintools import DarwinFile, MachOReference, DarwinFileTracker
-else:
-    from .patchelf import Patchelf
 
 __all__ = ["ConfigError", "ConstantsModule", "Executable", "Freezer"]
 
@@ -164,7 +162,7 @@ class Freezer(ABC):
             return
         self._create_directory(target.parent)
         if self.silent < 1:
-            print(f"copying {source!s} -> {target!s}")
+            print(f"copying {source} -> {target}")
         shutil.copyfile(source, target)
         shutil.copystat(source, target)
         if include_mode:
@@ -198,7 +196,7 @@ class Freezer(ABC):
             path = Path(path)
         if not path.is_dir():
             if self.silent < 1:
-                print(f"creating directory {path!s}")
+                print(f"creating directory {path}")
             path.mkdir(parents=True, exist_ok=True)
 
     def _freeze_executable(self, exe: Executable) -> None:
@@ -210,9 +208,9 @@ class Freezer(ABC):
         )
 
         # Ensure the copy of default python libraries
-        dependent_files = self._get_dependent_files(exe.base)
+        dependent_files = self.get_dependent_files(exe.base)
         if not dependent_files:
-            dependent_files = self._get_dependent_files(Path(sys.executable))
+            dependent_files = self.get_dependent_files(Path(sys.executable))
         python_libs = tuple(self._default_bin_includes())
         python_dirs = {Path(sys.base_exec_prefix), Path(sys.exec_prefix)}
         python_dirs.add(Path(sysconfig.get_config_var("srcdir")))  # Linux
@@ -233,8 +231,14 @@ class Freezer(ABC):
         self._platform_add_extra_dependencies(dependent_files)
 
         for source in dependent_files:
-            # Store dynamic libraries in appropriate location for platform
+            # Store dynamic libraries in appropriate location for platform.
             self._copy_top_dependency(source)
+            # Once copied, it should be deleted from the list to ensure
+            # it will not be copied again.
+            name = Path(source.name)
+            if name in self.bin_includes:
+                self.bin_includes.remove(name)
+                self.bin_excludes.append(name)
 
         target_path = self.targetdir / exe.target_name
         self._copy_file(
@@ -291,13 +295,6 @@ class Freezer(ABC):
         """Return the paths of directories which contain files that should
         be included."""
 
-    @abstractmethod
-    def _get_dependent_files(self, path: Path) -> Set[Path]:
-        """Return the file's dependencies using platform-specific tools (the
-        imagehlp library on Windows, otool on Mac OS X and ldd on Linux);
-        limit this list by the exclusion lists as needed.
-        (Implemented separately for each platform.)"""
-
     def _get_module_finder(self) -> ModuleFinder:
         finder = ModuleFinder(
             self.include_files,
@@ -321,7 +318,7 @@ class Freezer(ABC):
         """Platform-specific post-Freeze work."""
 
     def _print_report(self, filename: Path, modules: List[Module]) -> None:
-        print(f"writing zip file {filename!s}\n")
+        print(f"writing zip file {filename}\n")
         print("  {:<25} {}".format("Name", "File"))
         print("  {:<25} {}".format("----", "----"))
         for module in modules:
@@ -329,7 +326,7 @@ class Freezer(ABC):
                 print("P", end="")
             else:
                 print("m", end="")
-            print(f" {module.name:<25} {(module.file or '')!s}")
+            print(f" {module.name:<25} {module.file or ''}")
 
     @staticmethod
     def _remove_version_numbers(filename: str) -> str:
@@ -471,19 +468,24 @@ class Freezer(ABC):
             include_in_file_system = module.in_file_system
 
             # if the module refers to a package, check to see if this package
-            # should be included in the zip file or should be written to the
-            # file system; if the package should be written to the file system,
-            # any non-Python files are copied at this point if the target
-            # directory does not already exist
+            # should be written to the file system
             if (
-                include_in_file_system
+                include_in_file_system >= 1
                 and module.path is not None
                 and module.file is not None
             ):
                 parts = module.name.split(".")
                 target_package_dir = target_lib_dir.joinpath(*parts)
-                source_package_dir = module.file.parent
-                if not target_package_dir.exists():
+                if include_in_file_system == 2:
+                    # a few packages are optimized on the hooks,
+                    # so for now create the directory for this package
+                    self._create_directory(target_package_dir)
+
+                elif not target_package_dir.exists():
+                    # whether the package and its data will be written to the
+                    # file system, any non-Python files are copied at this
+                    # point if the target directory does not already exist
+                    source_package_dir = module.file.parent
                     if self.silent < 1:
                         print(f"copying data from package {module.name}...")
                     shutil.copytree(
@@ -512,7 +514,7 @@ class Freezer(ABC):
             if (
                 module.code is None
                 and module.file is not None
-                and not include_in_file_system
+                and include_in_file_system == 0
             ):
                 parts = module.name.split(".")[:-1]
                 parts.append(module.file.name)
@@ -537,7 +539,7 @@ class Freezer(ABC):
                 data = header + marshal.dumps(module.code)
 
             # if the module should be written to the file system, do so
-            if include_in_file_system and module.file is not None:
+            if include_in_file_system >= 1 and module.file is not None:
                 parts = module.name.split(".")
                 if module.code is None:
                     parts.pop()
@@ -606,9 +608,7 @@ class Freezer(ABC):
                 os.environ["PATH"] = origPath
 
     def Freeze(self):
-        self.dependent_files: Dict[Path, Set[Path]] = {}
         self.files_copied: Set[Path] = set()
-        self.linker_warnings: Dict[Path, Any] = {}
 
         finder: ModuleFinder = self._get_module_finder()
         self.finder: ModuleFinder = finder
@@ -648,9 +648,10 @@ class Freezer(ABC):
         self._post_freeze_hook()
 
 
-class WinFreezer(Freezer):
+class WinFreezer(Freezer, PEParser):
     def __init__(self, *args, **kwargs):
-        super().__init__(*args, **kwargs)
+        Freezer.__init__(self, *args, **kwargs)
+        PEParser.__init__(self, self.silent)
 
         # deal with C-runtime files
         self.runtime_files: Set[str] = set()
@@ -660,19 +661,33 @@ class WinFreezer(Freezer):
     def _add_resources(self, exe: Executable) -> None:
         target_path: Path = self.targetdir / exe.target_name
 
+        # Change the manifest
+        manifest: Optional[str] = exe.manifest
+        if manifest is not None or exe.uac_admin:
+            if self.silent < 1:
+                print(f"writing manifest -> {target_path}")
+            try:
+                if exe.uac_admin:
+                    manifest = manifest or self.read_manifest(target_path)
+                    manifest = manifest.replace(
+                        "asInvoker", "requireAdministrator"
+                    )
+                self.write_manifest(target_path, manifest)
+            except FileNotFoundError as exc:
+                if self.silent < 3:
+                    print("WARNING:", exc)
+            except RuntimeError as exc:
+                if self.silent < 3:
+                    print(f"WARNING: error parsing {target_path}:", exc)
+
         # Add version resource
         if self.metadata is not None:
-            warning_msg = "WARNING: unable to create version resource"
-            if version_stamp is None:
+            warning_msg = "WARNING: unable to create version resource:"
+            if not self.metadata.version:
                 if self.silent < 3:
-                    print(warning_msg)
-                    print("install pywin32 extensions first")
-            elif not self.metadata.version:
-                if self.silent < 3:
-                    print(warning_msg)
-                    print("version must be specified")
+                    print(warning_msg, "version must be specified")
             else:
-                versionInfo = VersionInfo(
+                version = VersionInfo(
                     self.metadata.version,
                     comments=self.metadata.long_description,
                     description=self.metadata.description,
@@ -680,13 +695,18 @@ class WinFreezer(Freezer):
                     product=self.metadata.name,
                     copyright=exe.copyright,
                     trademarks=exe.trademarks,
+                    verbose=bool(self.silent < 1),
                 )
-                version_stamp(str(target_path), versionInfo)
+                try:
+                    version.stamp(target_path)
+                except (FileNotFoundError, RuntimeError) as exc:
+                    if self.silent < 3:
+                        print(warning_msg, exc)
 
         # Add icon
         if exe.icon is not None:
             try:
-                winutil.AddIcon(target_path, exe.icon)
+                AddIcon(target_path, exe.icon)
             except MemoryError:
                 if self.silent < 3:
                     print("WARNING: MemoryError")
@@ -708,7 +728,7 @@ class WinFreezer(Freezer):
 
         # Update the PE checksum (or fix it in case it is zero)
         try:
-            winutil.UpdateCheckSum(target_path)
+            UpdateCheckSum(target_path)
         except MemoryError:
             if self.silent < 3:
                 print("WARNING: MemoryError")
@@ -720,7 +740,8 @@ class WinFreezer(Freezer):
         """Called for copying certain top dependencies in _freeze_executable.
         We need this as a separate method so that it can be overridden on
         Darwin and Windows."""
-        # top dependencies do not go into lib on windows.
+        # top dependencies do not go into lib on windows, but the C runtimes
+        # can be handled in _copy_file/_pre_copy_hook
         target = self.targetdir / source.name
         self._copy_file(
             source, target, copy_dependent_files=True, include_mode=True
@@ -757,7 +778,7 @@ class WinFreezer(Freezer):
         ):
 
             targetdir = target.parent
-            for dependent_file in self._get_dependent_files(source):
+            for dependent_file in self.get_dependent_files(source):
                 target = targetdir / dependent_file.name
                 self._copy_file(dependent_file, target, copy_dependent_files)
 
@@ -778,8 +799,8 @@ class WinFreezer(Freezer):
         return python_shared_libs
 
     def _default_bin_path_excludes(self) -> List[str]:
-        systemDir = winutil.GetSystemDir()
-        windowsDir = winutil.GetWindowsDir()
+        systemDir = GetSystemDir()
+        windowsDir = GetWindowsDir()
         return [windowsDir, systemDir, os.path.join(windowsDir, "WinSxS")]
 
     def _default_bin_path_includes(self) -> List[str]:
@@ -793,32 +814,6 @@ class WinFreezer(Freezer):
         # return only valid paths
         return [str(path) for path in paths if path.is_dir()]
 
-    def _get_dependent_files(self, path: Path) -> Set[Path]:
-        try:
-            return self.dependent_files[path]
-        except KeyError:
-            pass
-
-        dependent_files: Set[Path] = set()
-        if path.suffix.lower().endswith((".exe", ".dll", ".pyd")):
-            origPath = os.environ["PATH"]
-            os.environ["PATH"] = (
-                origPath + os.pathsep + os.pathsep.join(sys.path)
-            )
-            try:
-                files: List[str] = winutil.GetDependentFiles(path)
-            except winutil.BindError as exc:
-                # Sometimes this gets called when path is not actually
-                # a library (See issue 88).
-                if self.silent < 3:
-                    print("WARNING: ignoring error during ", end="")
-                    print(f"GetDependentFiles({path}):", exc)
-            else:
-                dependent_files = {Path(dep) for dep in files}
-            os.environ["PATH"] = origPath
-        self.dependent_files[path] = dependent_files
-        return dependent_files
-
     def _platform_add_extra_dependencies(
         self, dependent_files: Set[Path]
     ) -> None:
@@ -853,9 +848,10 @@ class WinFreezer(Freezer):
             self.bin_excludes.extend([Path(name) for name in winmsvcr.FILES])
 
 
-class DarwinFreezer(Freezer):
+class DarwinFreezer(Freezer, Parser):
     def __init__(self, *args, **kwargs):
-        super().__init__(*args, **kwargs)
+        Freezer.__init__(self, *args, **kwargs)
+        Parser.__init__(self, self.silent)
         self.darwinTracker: Optional[DarwinFileTracker] = None
         self.darwinTracker = DarwinFileTracker()
 
@@ -894,7 +890,7 @@ class DarwinFreezer(Freezer):
             # Always copy dependent files on root directory
             # to allow to set relative reference
             targetdir = self.targetdir
-            for dependent in self._get_dependent_files(source, darwin_file):
+            for dependent in self.get_dependent_files(source, darwin_file):
                 target = targetdir / dependent.name
                 reference = darwin_file.getMachOReferenceForPath(dependent)
                 self._copy_file_recursion(
@@ -934,7 +930,7 @@ class DarwinFreezer(Freezer):
             return
         self._create_directory(target.parent)
         if self.silent < 1:
-            print(f"copying {source!s} -> {target!s}")
+            print(f"copying {source} -> {target}")
         shutil.copyfile(source, target)
         shutil.copystat(source, target)
         if include_mode:
@@ -958,7 +954,7 @@ class DarwinFreezer(Freezer):
         target = self.targetdir / "lib" / source.name
 
         # this recovers the cached MachOReference pointers to the files
-        # found by the _get_dependent_files calls made previously (if any).
+        # found by the get_dependent_files calls made previously (if any).
         # If one is found, pass into _copy_file.
         # We need to do this so the file knows what file referenced it,
         # and can therefore calculate the appropriate rpath.
@@ -978,7 +974,7 @@ class DarwinFreezer(Freezer):
     def _default_bin_path_includes(self) -> List[str]:
         return [sysconfig.get_config_var("DESTSHARED")]
 
-    def _get_dependent_files(
+    def get_dependent_files(
         self, path: Path, darwinFile: Optional["DarwinFile"] = None
     ) -> Set[Path]:
         try:
@@ -986,7 +982,7 @@ class DarwinFreezer(Freezer):
         except KeyError:
             pass
 
-        # if darwinFile is None (which means that _get_dependent_files is
+        # if darwinFile is None (which means that get_dependent_files is
         # being called outside of _copy_file -- e.g., one of the
         # preliminary calls in _freeze_executable), create a temporary
         # DarwinFile object for the path, just so we can read its
@@ -1007,13 +1003,26 @@ class DarwinFreezer(Freezer):
         return dependent_files
 
 
-class LinuxFreezer(Freezer):
+class LinuxFreezer(Freezer, ELFParser):
     def __init__(self, *args, **kwargs):
-        super().__init__(*args, **kwargs)
-        self.patchelf = Patchelf()
+        Freezer.__init__(self, *args, **kwargs)
+        ELFParser.__init__(self, self.silent)
+        self._symlinks: Set[Tuple[Path, str]] = set()
 
     def _pre_copy_hook(self, source: Path, target: Path) -> Tuple[Path, Path]:
-        """Prepare the source and target paths."""
+        """Prepare the source and target paths. Also, ensures that the source
+        of a symlink is copied deferring the link creation."""
+        if source.is_symlink():
+            real_source = source.resolve()
+            symlink = os.readlink(source)
+            real_target = (target.parent / symlink).resolve()
+            try:
+                real_target.relative_to(self.targetdir / "lib")
+            except ValueError:
+                symlink = real_source.name
+                real_target = target.with_name(symlink)
+            self._symlinks.add((target, symlink))
+            return real_source, real_target
         return source, target
 
     def _post_copy_hook(
@@ -1031,7 +1040,7 @@ class LinuxFreezer(Freezer):
             source_dir = source.parent
             library_dir = self.targetdir / "lib"
             fix_rpath = set()
-            for dependent_file in self._get_dependent_files(source):
+            for dependent_file in self.get_dependent_files(source):
                 if not self._should_copy_file(dependent_file):
                     continue
                 try:
@@ -1048,22 +1057,35 @@ class LinuxFreezer(Freezer):
                                 dependent_target = library_dir / relative
                                 break
                 else:
-                    if targetdir == library_dir:
-                        parts = relative.parts
-                        while parts[0] == os.pardir:
-                            parts = parts[1:]
-                        relative = Path(*parts)
                     dependent_target = targetdir / relative
+                    dependent_target = dependent_target.resolve()
+                    try:
+                        dependent_target.relative_to(library_dir)
+                    except ValueError:
+                        parts = list(relative.parts)
+                        while parts[0] == os.pardir:
+                            parts.pop(0)
+                        relative = Path(*parts)
+                        dependent_target = library_dir / relative
                 dep_libs = str(relative.parent)
                 fix_rpath.add(dep_libs)
                 self._copy_file(
                     dependent_file, dependent_target, copy_dependent_files
                 )
             if fix_rpath:
-                has_rpath = self.patchelf.get_rpath(target)
+                has_rpath = self.get_rpath(target)
                 rpath = ":".join(f"$ORIGIN/{r}" for r in fix_rpath)
                 if has_rpath != rpath:
-                    self.patchelf.set_rpath(target, rpath)
+                    self.set_rpath(target, rpath)
+
+    def _post_freeze_hook(self):
+        target: Path
+        symlink: str
+        for target, symlink in self._symlinks:
+            if self.silent < 1:
+                print(f"linking {target} -> {symlink}")
+            if not target.exists():
+                target.symlink_to(symlink)
 
     def _copy_top_dependency(self, source: Path) -> None:
         """Called for copying certain top dependencies in _freeze_executable."""
@@ -1085,15 +1107,3 @@ class LinuxFreezer(Freezer):
     def _default_bin_path_includes(self) -> List[str]:
         # add the stdlib/lib-dynload directory
         return [sysconfig.get_config_var("DESTSHARED")]
-
-    def _get_dependent_files(self, path: Path) -> Set[Path]:
-        try:
-            return self.dependent_files[path]
-        except KeyError:
-            pass
-
-        dependent_files: Set[Path] = self.patchelf.get_needed(
-            path, self.linker_warnings, show_warnings=self.silent < 3
-        )
-        self.dependent_files[path] = dependent_files
-        return dependent_files
