@@ -9,7 +9,7 @@
 #define WIN32_LEAN_AND_MEAN
 #include <windows.h>
 #include <Winsvc.h>
-#include <shlwapi.h>
+#define OLECHAR WCHAR
 #include <cx_Logging.h>
 
 // define constants
